import tango
from tango import DeviceProxy
from tangostationcontrol.common.antennas import antenna_field_family_name
from tangostationcontrol.common.proxies.proxy import create_device_proxy
import logging

logger = logging.getLogger()


class AntennaDeviceProxyFactory:

    @staticmethod
    def create_device_proxy_for_antennafield(
        antennafield_name: str, write_access: bool = False
    ) -> DeviceProxy:
        """Get the device proxy for a given antenna field, if it exists."""
        try:
            family = antenna_field_family_name(antennafield_name)
            db = tango.Database()
            device_exported = db.get_device_exported_for_class(family)
            logger.info(
                "Creating device proxy for FIELD=%s FAMILY=%s",
                antennafield_name,
                family,
            )
            for device_name in device_exported:
                _, _, exported_antennafield_name = device_name.split("/")
                logger.info(
                    "Checking device FIELD=%s FAMILY=%s DEVICE_NAME=%s EXPORTED_FIELD=%s",
                    antennafield_name,
                    family,
                    device_name,
                    exported_antennafield_name,
                )
                if (
                    exported_antennafield_name.casefold()
                    == antennafield_name.casefold()
                ):
                    logger.info(
                        "Match found for FIELD=%s FAMILY=%s DEVICE_NAME=%s",
                        antennafield_name,
                        family,
                        device_name,
                    )
                    return create_device_proxy(device_name, write_access=write_access)
        except Exception as ex:
            logger.exception(
                "Failed to create device proxy",
                extra={"antennafield_name": antennafield_name},
            )
            raise IOError(
                f"Failed to create device proxy to '{antennafield_name}'"
            ) from ex

        raise ValueError(f"Antenna field '{antennafield_name}' not found")
